@@ -49,7 +49,7 @@
  * Allow libapparmor to build on older glibcs and other libcs that do
  * not support reallocarray.
  */
-#ifndef HAVE_REALLOCARRY
+#ifndef HAVE_REALLOCARRAY
 void *reallocarray(void *ptr, size_t nmemb, size_t size)
 {
 	return realloc(ptr, nmemb * size);
@@ -126,7 +126,7 @@ bool atomic_dec_and_test(unsigned int *v)
 
 int _aa_is_blacklisted(const char *name)
 {
-	size_t name_len = strlen(name);
+	ssize_t name_len = strlen(name);
 	struct ignored_suffix_t *suffix;
 
 	/* skip dot files and files with no name */
@@ -315,8 +315,7 @@ static ssize_t readdirfd(int dirfd, struct dirent ***out,
 			  int (*dircmp)(const struct dirent **, const struct dirent **))
 {
 	struct dirent **dents = NULL, *dent;
-	ssize_t n = 0;
-	size_t i;
+	ssize_t n = 0, i;
 	int save;
 	DIR *dir;
 
@@ -389,7 +388,7 @@ int _aa_overlaydirat_for_each(int dirfd[], int n, void *data,
 {
 	autofree struct dirent **list = NULL;
 	autofree struct overlaydir *overlay = NULL;
-	int i, k;
+	int i;
 	int n_list, size = 0, max_size = 0;
 	int rc = 0;
 
@@ -400,10 +399,10 @@ int _aa_overlaydirat_for_each(int dirfd[], int n, void *data,
 			return -1;
 		}
 		if (merge(overlay, size, max_size, list, n_list, dirfd[i])) {
-			for (k = 0; k < n_list; k++)
-				free(list[k]);
-			for (k = 0; k < size; k++)
-				free(overlay[k].dent);
+			for (i = 0; i < n_list; i++)
+				free(list[i]);
+			for (i = 0; i < size; i++)
+				free(overlay[i].dent);
 			return -1;
 		}
 	}
