@@ -22,7 +22,6 @@
 
 #include <iomanip>
 #include <string>
-#include <iostream>
 #include <sstream>
 
 #include "network.h"
@@ -30,6 +29,9 @@
 #include "profile.h"
 #include "af_unix.h"
 
+/* See unix(7) for autobind address definiation */
+#define autobind_address_pattern "\\x00[0-9a-f][0-9a-f][0-9a-f][0-9a-f][0-9a-f]";
+
 int parse_unix_mode(const char *str_mode, int *mode, int fail)
 {
 	return parse_X_mode("unix", AA_VALID_NET_PERMS, str_mode, mode, fail);
@@ -55,7 +57,9 @@ void unix_rule::move_conditionals(struct cond_entry *conds)
 		}
 		if (strcmp(ent->name, "addr") == 0) {
 			move_conditional_value("unix socket", &addr, ent);
-			if (addr[0] != '@' && strcmp(addr, "none") != 0)
+			if (addr[0] != '@' &&
+			    !(strcmp(addr, "none") == 0 ||
+			      strcmp(addr, "auto") == 0))
 				yyerror("unix rule: invalid value for addr='%s'\n", addr);
 		}
 
@@ -83,7 +87,9 @@ void unix_rule::move_peer_conditionals(struct cond_entry *conds)
 		}
 		if (strcmp(ent->name, "addr") == 0) {
 			move_conditional_value("unix", &peer_addr, ent);
-			if (peer_addr[0] != '@' && strcmp(peer_addr, "none") != 0)
+			if ((peer_addr[0] != '@') &&
+			    !(strcmp(peer_addr, "none") == 0 ||
+			      strcmp(peer_addr, "auto") == 0))
 				yyerror("unix rule: invalid value for addr='%s'\n", peer_addr);
 		}
 	}
@@ -160,26 +166,10 @@ int unix_rule::expand_variables(void)
 	return 0;
 }
 
-/* do we want to warn once/profile or just once per compile?? */
-static void warn_once(const char *name, const char *msg)
-{
-	static const char *warned_name = NULL;
 
-	if (warned_name != name) {
-		cerr << "Warning from profile " << name << " (";
-		if (current_filename)
-			cerr << current_filename;
-		else
-			cerr << "stdin";
-		cerr << "): " << msg << "\n";
-		warned_name = name;
-	}
-}
-
-static void warn_once(const char *name)
+void unix_rule::warn_once(const char *name)
 {
-	if (warnflags & WARN_RULE_NOT_ENFORCED)
-		warn_once(name, "extended network unix socket rules not enforced");
+	rule_t::warn_once(name, "extended network unix socket rules not enforced");
 }
 
 static void writeu16(std::ostringstream &o, int v)
@@ -215,17 +205,11 @@ void unix_rule::downgrade_rule(Profile &prof) {
 	}
 }
 
-static uint32_t map_perms(uint32_t mask)
-{
-	return (mask & 0x7f) |
-		((mask & (AA_NET_GETATTR | AA_NET_SETATTR)) << (AA_OTHER_SHIFT - 8)) |
-		((mask & (AA_NET_ACCEPT | AA_NET_BIND | AA_NET_LISTEN)) >> 4) | /* 2 + (AA_OTHER_SHIFT - 20) */
-		((mask & (AA_NET_SETOPT | AA_NET_GETOPT)) >> 5); /* 5 + (AA_OTHER_SHIFT - 24) */
-}
-
 void unix_rule::write_to_prot(std::ostringstream &buffer)
 {
-	buffer << "\\x" << std::setfill('0') << std::setw(2) << std::hex << AA_CLASS_NET;
+	int c = features_supports_networkv8 ? AA_CLASS_NETV8 : AA_CLASS_NET;
+
+	buffer << "\\x" << std::setfill('0') << std::setw(2) << std::hex << c;
 	writeu16(buffer, AF_UNIX);
 	if (sock_type)
 		writeu16(buffer, sock_type_n);
@@ -247,6 +231,12 @@ bool unix_rule::write_addr(std::ostringstream &buffer, const char *addr)
 		if (strcmp(addr, "none") == 0) {
 			/* anonymous */
 			buffer << "\\x01";
+		} else if (strcmp(addr, "auto") == 0) {
+			/* autobind - special autobind rule written already
+			 * just generate pattern that matches autobind
+			 * generated addresses.
+			 */
+			buffer << autobind_address_pattern;
 		} else {
 			/* skip leading @ */
 			ptype = convert_aaregex_to_pcre(addr + 1, 0, glob_null, buf, &pos);
@@ -324,12 +314,12 @@ int unix_rule::gen_policy_re(Profile &prof)
 	 * rules ability
 	 */
 	downgrade_rule(prof);
-	if (!kernel_supports_unix) {
-		if (kernel_supports_network) {
+	if (!features_supports_unix) {
+		if (features_supports_network || features_supports_networkv8) {
 			/* only warn if we are building against a kernel
 			 * that requires downgrading */
 			if (warnflags & WARN_RULE_DOWNGRADED)
-				warn_once(prof.name, "downgrading extended network unix socket rule to generic network rule\n");
+				rule_t::warn_once(prof.name, "downgrading extended network unix socket rule to generic network rule\n");
 			/* TODO: add ability to abort instead of downgrade */
 			return RULE_OK;
 		}
@@ -348,6 +338,33 @@ int unix_rule::gen_policy_re(Profile &prof)
 		mask &= ~AA_NET_CREATE;
 	}
 
+	/* write special pattern for autobind? Will not grant bind
+	 * on any specific address
+	 */
+	if ((mask & AA_NET_BIND) && (!addr || (strcmp(addr, "auto") == 0))) {
+		std::ostringstream tmp;
+
+		tmp << buffer.str();
+		/* todo: change to out of band separator */
+		/* skip addr, its 0 length */
+		tmp << "\\x00";
+		/* local label option */
+		if (!write_label(tmp, label))
+			goto fail;
+		/* seperator */
+		tmp << "\\x00";
+
+		buf = tmp.str();
+		if (!prof.policy.rules->add_rule(buf.c_str(), deny,
+						 map_perms(AA_NET_BIND),
+						 map_perms(audit & AA_NET_BIND),
+						 dfaflags))
+			goto fail;
+		/* clear if auto, else generic need to generate addr below */
+		if (addr)
+			mask &= ~AA_NET_BIND;
+	}
+
 	if (mask) {
 		/* local addr */
 		if (!write_addr(buffer, addr))
