@@ -173,9 +173,9 @@ void post_process_file_entries(Profile *prof)
 	 * access to /proc/self/attr/current
 	 */
 	if (cp_mode & AA_CHANGE_PROFILE) {
-		/* FIXME: should use @{PROC}/@{PID}/attr/{current,exec} */
+		/* FIXME: should use @{PROC}/@{PID}/attr/{apparmor/,}{current,exec} */
 		struct cod_entry *new_ent;
-		char *buffer = strdup("/proc/*/attr/{current,exec}");
+		char *buffer = strdup("/proc/*/attr/{apparmor/,}{current,exec}");
 		if (!buffer) {
 			PERROR("Memory allocation error\n");
 			exit(1);
@@ -196,7 +196,7 @@ void post_process_rule_entries(Profile *prof)
 }
 
 
-#define CHANGEHAT_PATH "/proc/[0-9]*/attr/current"
+#define CHANGEHAT_PATH "/proc/[0-9]*/attr/{apparmor/,}current"
 
 /* add file rules to access /proc files to call change_hat()
  */
@@ -205,7 +205,7 @@ static int profile_add_hat_rules(Profile *prof)
 	struct cod_entry *entry;
 
 	/* don't add hat rules if not hat or profile doesn't have hats */
-	if (!prof->flags.hat || !prof->hat_table.empty())
+	if (!prof->flags.hat && prof->hat_table.empty())
 		return 0;
 
 	/* add entry to hat */
@@ -302,7 +302,12 @@ Profile *merge_policy(Profile *a, Profile *b)
 	}
 	b->entries = NULL;
 
-	a->flags.complain = a->flags.complain || b->flags.complain;
+	if (merge_profile_mode(a->flags.mode, b->flags.mode) == MODE_CONFLICT) {
+		PERROR("ASSERT: policy merge with different modes 0x%x != 0x%x\n",
+		       a->flags.mode, b->flags.mode);
+		exit(1);
+	}
+
 	a->flags.audit = a->flags.audit || b->flags.audit;
 
 	a->caps.allow |= b->caps.allow;
