@@ -70,7 +70,7 @@ setup (name='apparmor',
        long_description='Python libraries for AppArmor utilities',
        author='AppArmor Developers',
        author_email='apparmor@lists.ubuntu.com',
-       url='https://launchpad.net/apparmor',
+       url='https://gitlab.com/apparmor/apparmor',
        license='GPL-2',
        cmdclass={'install': Install},
        package_dir={'apparmor': 'staging'},
